@@ -40,7 +40,7 @@
 #define BACKLOG 1024     // how many pending connections queue will hold
 #define MAX_BUFF 32768  //read buffer size. bigger faster, but memory foot print bigger
 #define MAX_RETRY 9   //number of connection retry
-#define VERSION "0.4"
+#define VERSION "0.5"
 
 /*
 Structure we use for each client's connection. 
@@ -65,6 +65,7 @@ struct client {
         char *response_header;
 	int response_header_length;
         PyObject *response_content;
+        PyObject *response_content_obj;
         FILE *response_fp; // file of the sent file
 };
 
@@ -73,7 +74,7 @@ struct client {
 Somme  global variables
 */
 char *server_name="127.0.0.1";
-short server_port=8000;
+char *server_port="8000";
 int sockfd;  // main sock_fd
 int debug=0; //1 full debug detail: 0 nodebug
 
@@ -116,6 +117,10 @@ void close_connection(struct client *cli)
     free(cli->uri_path);
     //free(cli->response_header);
     Py_XDECREF(cli->response_content);
+    if (cli->response_content_obj!=NULL)
+    {
+        Py_DECREF(cli->response_content_obj);
+    }
     if (cli->response_fp){
         //free(cli->response_fp);
     }
@@ -228,7 +233,6 @@ char * remove_leading_and_trailing_spaces(char* s)
 }
 
 
-
 /*
 This procedure transform the header to something required by wsgi.
 */
@@ -304,9 +308,6 @@ header_to_dict(struct client *cli)
     cli->protocol=(char *)calloc(strlen(protocol)+1, sizeof(char));
      assert(cli->protocol);
      strcpy(cli->protocol, protocol);   // will be cleaned with cli
-     pyval = Py_BuildValue("s", protocol );
-     PyDict_SetItemString(pydict, "wsgi.url_scheme", pyval); // can be http or https
-     Py_DECREF(pyval);
     pyval = Py_BuildValue("s", http_version );
      PyDict_SetItemString(pydict, "fapws.major_minor", pyval); // can be 0.9, 1.0 or 1.1
      Py_DECREF(pyval);
@@ -315,7 +316,7 @@ header_to_dict(struct client *cli)
     pyval = Py_BuildValue("s", server_name );
     PyDict_SetItemString(pydict, "SERVER_NAME", pyval);
     Py_DECREF(pyval);
-    pyval = Py_BuildValue("i", server_port );
+    pyval = Py_BuildValue("s", server_port );
     PyDict_SetItemString(pydict, "SERVER_PORT", pyval);
     Py_DECREF(pyval);
     //then we analyze all the other lines up to the body (not included) of the header 
@@ -389,7 +390,7 @@ py_build_method_variables( struct client *cli)
     pydummy=PyString_FromString(server_name);
     PyDict_SetItemString(pydict, "SERVER_NAME", pydummy);
     Py_DECREF(pydummy);
-    pydummy=Py_BuildValue("h", server_port);
+    pydummy=PyString_FromString(server_port);
     PyDict_SetItemString(pydict, "SERVER_PORT", pydummy);
     Py_DECREF(pydummy);
     pydummy=PyString_FromString(cli->uri);
@@ -446,6 +447,7 @@ py_get_request_info(struct client *cli)
     
     pydummy=PyString_FromString(cli->remote_addr);
     PyDict_SetItemString(pydict, "fapws.remote_addr", pydummy);
+    PyDict_SetItemString(pydict, "REMOTE_ADDR", pydummy);
     Py_DECREF(pydummy);
     pydummy=Py_BuildValue("H", cli->remote_port);
     PyDict_SetItemString(pydict, "fapws.remote_port", pydummy);
@@ -460,12 +462,20 @@ py_get_request_info(struct client *cli)
 /*
 In case of POST, we must populate some additional wsgi parameters.
 */
-static void 
+static int  
 manage_header_body(struct client *cli, PyObject *pyenviron)
 {
     PyObject *pydummy; 
 
     pydummy=PyDict_GetItemString(pyenviron,"HTTP_CONTENT_LENGTH");
+    if (pydummy==NULL) {
+        //a POST without content-length is not a valid 
+        if (debug) {
+            printf("We cannot manage a POST without Content-Length\n");
+            printf("Associated header:\n%s\n",cli->input_header);
+        }
+        return -411;
+    }
     char *content_length_str = PyString_AsString(pydummy);
     int content_length = atoi(content_length_str);
     pydummy = PyInt_FromString(content_length_str, NULL, 10);
@@ -505,6 +515,7 @@ manage_header_body(struct client *cli, PyObject *pyenviron)
             Py_DECREF(pydummy);
         }
     }
+    return 0;
 }
 
 /*
@@ -518,6 +529,7 @@ static int
 python_handler(struct client *cli)
 {
     PyObject *pydict, *pydummy;
+    int ret;
 
     if (debug)
          printf("host=%s,port=%i:python_handler:HEADER:\n%s**\n", cli->remote_addr, cli->remote_port, cli->input_header);
@@ -540,7 +552,7 @@ python_handler(struct client *cli)
     if (pydict==Py_None)
     {
         Py_DECREF(pyenviron);
-        return -1;
+        return -500;
     }
     update_environ(pyenviron, pydict, "update_headers");
     Py_DECREF(pydict);   
@@ -553,7 +565,7 @@ python_handler(struct client *cli)
         Py_DECREF(pysupportedhttpcmd);
         Py_DECREF(pydummy);
         Py_DECREF(pyenviron);
-        return -2;
+        return -501;
     }
     Py_DECREF(pydummy);
     //  2ter) we treat directly the OPTIONS command
@@ -603,7 +615,11 @@ python_handler(struct client *cli)
         if (strcmp(cli->cmd,"POST")==0)
         {
             //printf("manage the post\n");
-            manage_header_body(cli, pyenviron);
+            ret=manage_header_body(cli, pyenviron);
+            //printf("manage header return:%i\n",ret);
+            if (ret < 0) {
+                return ret;
+            }
         }
     }
     //  6) add some request info
@@ -617,6 +633,14 @@ python_handler(struct client *cli)
     // 8) execute python callbacks with his parameters
     PyObject *pyarglist = Py_BuildValue("(OO)", pyenviron, pystart_response );
     cli->response_content = PyEval_CallObject(cli->wsgi_cb,pyarglist);
+    if (cli->response_content!=NULL) 
+    {
+        if ((PyFile_Check(cli->response_content)==0) && (PyIter_Check(cli->response_content)==1)) {
+            //This is an Iterator object. We have to execute it first
+            cli->response_content_obj = cli->response_content;
+            cli->response_content = PyIter_Next(cli->response_content_obj);
+        }
+    }
     Py_DECREF(pyarglist);
     Py_XDECREF(cli->wsgi_cb);
     if (cli->response_content!=NULL) 
@@ -746,21 +770,27 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     if (cli->response_iter_sent==-2)
     { 
         //we must send an header or an error
-        if ((ret=python_handler(cli))==0) //look for python callback and execute it
+        ret=python_handler(cli); //look for python callback and execute it
+        if (ret==0) //look for python callback and execute it
         {
             //uri not found
             response="HTTP/1.1 500 Not found\r\nContent-Type: text/html\r\nServer: fapws3/" VERSION "\r\n\r\n<html><head><title>Page not found</head><body><p>Page not found!!!</p></body></html>";
             write_cli(cli,response, strlen(response), revents);
             stop=1;
         } 
-        else if (ret==-1)
+        else if (ret==-411)
         {
-            //problem to parse the request
-            response="HTTP/1.1 400 Bad Request\r\nContent-Type: text/html\r\nServer: fapws3/" VERSION "\r\n\r\n<html><head><title>Bad request</head><body><p>Bad request!!!</p></body></html>";
+            response="HTTP/1.1 411 Length Required\r\nContent-Type: text/html\r\nServer: fapws3/" VERSION "\r\n\r\n<html><head><title>Length Required</head><body><p>Length Required!!!</p></body></html>";
             write_cli(cli,response, strlen(response), revents);
             stop=1;
         }
-        else if (ret==-2)
+        else if (ret==-500)
+        {
+            response="HTTP/1.1 500 Internal Server Error\r\nContent-Type: text/html\r\nServer: fapws3/" VERSION "\r\n\r\n<html><head><title>Internal Server Error</head><body><p>Internal Server Error!!!</p></body></html>";
+            write_cli(cli,response, strlen(response), revents);
+            stop=1;
+        }
+        else if (ret==-501)
         {
             //problem to parse the request
             response="HTTP/1.1 501 Not Implemented\r\nContent-Type: text/html\r\nServer: fapws3/" VERSION "\r\n\r\n<html><head><title>Not Implemented</head><body><p>Not Implemented!!!</p></body></html>";
@@ -782,7 +812,7 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     else 
     {
         //we let the python developer to manage other HTTP command
-        if (PyList_Check(cli->response_content)) //we treat list object
+        if ((PyList_Check(cli->response_content))  && (cli->response_content_obj==NULL)) //we treat list object
         {
             cli->response_iter_sent++;
             if (cli->response_iter_sent<PyList_Size(cli->response_content)) 
@@ -812,7 +842,7 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
                 stop=2;
             }    
         } 
-        else if (PyFile_Check(cli->response_content)) // we treat file object
+        else if (PyFile_Check(cli->response_content) && (cli->response_content_obj==NULL)) // we treat file object
         {
             if (cli->response_iter_sent==-1) // we need to initialise the file descriptor
             {
@@ -839,10 +869,10 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
             }
             //free(buff);
         } 
-        else if (PyIter_Check(cli->response_content)) //we treat Iterator object
+        else if (PyIter_Check(cli->response_content_obj)) //we treat Iterator object
         {
             cli->response_iter_sent++;
-            PyObject *pyelem = PyIter_Next(cli->response_content);
+            PyObject *pyelem = cli->response_content;
             if (pyelem == NULL) 
             {
                 stop = 2;
@@ -869,6 +899,15 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
                     printf("The item %i of your iterator is not a string!!!!  It will be skipped\n",cli->response_iter_sent);
                 }
                 Py_DECREF(pyelem);
+                cli->response_content = PyIter_Next(cli->response_content_obj);
+                if (cli->response_content==NULL)
+                {
+                     if (debug)
+                     {
+                         printf("host=%s,port=%i iterator ended uri=%s\n", cli->remote_addr, cli->remote_port, cli->uri );
+                     }
+                     stop=2;
+                }
             }    
         } 
         else 
@@ -880,14 +919,16 @@ static void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     }// end of GET OR POST request
     if (stop==2)
     {
+      if (cli->response_content!=NULL) {
         if (PyObject_HasAttrString(cli->response_content, "close"))
         {
             PyObject *pydummy=PyObject_GetAttrString(cli->response_content, "close");
             PyObject_CallFunction(pydummy, NULL);
             Py_DECREF(pydummy);
         }
-        ev_io_stop(EV_A_ w);
-        close_connection(cli);
+      }
+      ev_io_stop(EV_A_ w);
+      close_connection(cli);
     }
     if (stop==1)
     {
@@ -1002,6 +1043,7 @@ static void accept_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     cli->wsgi_cb=NULL;
     cli->response_header=NULL;
     cli->response_content=NULL;
+    cli->response_content_obj=NULL;
     if (debug)
         printf("host:%s,port:%i accept_cb: cli:%p, input_header:%p\n", inet_ntoa (client_addr.sin_addr),ntohs(client_addr.sin_port), cli, cli->input_header);
     cli->input_pos=0;
@@ -1049,15 +1091,13 @@ http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
 //    struct sockaddr_storage their_addr; // connector's address information
     int yes=1;
     int rv;
-    char strport[7];
 
-    PyArg_ParseTuple(args, "si", &server_name, &server_port);
+    PyArg_ParseTuple(args, "ss", &server_name, &server_port);
     memset(&hints, 0, sizeof hints);
     hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_flags = AI_PASSIVE; // use my IP
-    snprintf(strport, sizeof (strport), "%d", server_port);
-    if ((rv = getaddrinfo(server_name, strport, &hints, &servinfo)) == -1) {
+    if ((rv = getaddrinfo(server_name, server_port, &hints, &servinfo)) == -1) {
         fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(rv));
         exit(1);
     }
@@ -1096,7 +1136,7 @@ http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
         perror("listen");
         exit(1);
     }
-    printf("listen on %s:%i\n", server_name, server_port);
+    printf("listen on %s:%s\n", server_name, server_port);
     return Py_None;
 }
 
